# -*- coding: utf-8 -*-

"""Core components for solving issues with anaconda config."""

from __future__ import annotations

__all__ = ['SolvedError', 'SolverCollection', 'POOL']

import typing
import binstar_client
from anaconda_navigator.utils import solvers as common_solvers

if typing.TYPE_CHECKING:
    import typing_extensions

    class Solver(typing_extensions.Protocol):  # pylint: disable=too-few-public-methods
        """
        Common interface for issue solvers.

        Each solver should:

        - check if there is an issue
        - perform corresponding actions to fix the issue
        - report what was wrong and what was changed
        """

        def __call__(self, configuration: typing.Any) -> typing.Union[None, SolvedError, typing.Iterable[SolvedError]]:
            """
            Solve an issue if it is detected.

            :param configuration: Anaconda client configuration to check for issues and fix.
            :return: Message(s) about what was fixed.
            """


class SolvedError(typing.NamedTuple):
    """Details on solved issue."""

    message: str


class SolverCollection(common_solvers.SolverCollection['Solver']):
    """Collection of issue solvers."""

    __slots__ = ('__errors',)

    def __init__(self) -> None:
        """Initialize new :class:`~SolverCollection` instance."""
        super().__init__()
        self.__errors: 'typing_extensions.Final[typing.List[SolvedError]]' = []

    def errors(self) -> typing.Iterator[SolvedError]:
        """
        Iterate through errors that were solved by solvers from this pool.

        Each error description is removed as soon as it is retrieved with this method.
        """
        while self.__errors:
            yield self.__errors.pop(0)

    def solve(
            self,
            *,
            tags: typing.Union[None, str, typing.Iterable[str]] = None,
    ) -> typing.Iterator[SolvedError]:
        """
        Detect and solve issues for the `context`.

        :param tags: Limit to issue solvers with specific tags.
        :return: Iterator of details about solved issues.
        """
        configuration: typing.Any = binstar_client.utils.get_config()

        solver_record: common_solvers.SolverRecord['Solver']
        for solver_record in self.only(tags=tags):
            result: typing.Union[None, SolvedError, typing.Iterable[SolvedError]] = solver_record.solver(configuration)
            if result is None:
                continue

            if isinstance(result, SolvedError):
                result = (result,)

            solved_error: SolvedError
            for solved_error in result:
                self.__errors.append(solved_error)
                yield solved_error

        binstar_client.utils.set_config(configuration)


POOL: 'typing_extensions.Final[SolverCollection]' = SolverCollection()



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Utils for login dialogs."""

import typing
from qtpy import QtCore
from qtpy import QtGui

EMAIL_RE = QtCore.QRegExp(r'^[a-zA-Z0-9_.+-]+@[a-zA-Z0-9-]+\.[a-zA-Z0-9-.]+$')
EMAIL_RE_VALIDATOR = QtGui.QRegExpValidator(EMAIL_RE)

USER_RE = QtCore.QRegExp(r'^[A-Za-z0-9_\.][A-Za-z0-9_\.\-]+$')
USER_RE_VALIDATOR = QtGui.QRegExpValidator(USER_RE)

FORGOT_LOGIN_URL = 'account/forgot_username'
FORGOT_PASSWORD_URL = 'account/forgot_password'  # nosec


class TextContainer(typing.NamedTuple):  # pylint: disable=missing-class-docstring

    title: typing.Optional[str] = None
    header_frame_logo_path: typing.Optional[str] = None
    info_frame_text: typing.Optional[str] = None
    info_frame_logo_path: typing.Optional[str] = None
    form_forgot_links_msg: typing.Optional[str] = None
    form_primary_text: typing.Optional[str] = None
    form_secondary_text: typing.Optional[str] = None
    form_input_label_text: typing.Optional[str] = None
    form_submit_button_text: typing.Optional[str] = None
    message_box_error_text: typing.Optional[str] = None



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors
#
# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)
# -----------------------------------------------------------------------------

"""Miscellaneous utilities."""

import os
import os.path as osp
import shutil
import uuid
import psutil
from anaconda_navigator.config import LOCKFILE, PIDFILE


def __remove_pyc_pyo(fname):
    """Eventually remove .pyc and .pyo files associated to a Python script"""
    if osp.splitext(fname)[1] == '.py':
        for ending in ('c', 'o'):
            if osp.exists(fname + ending):
                os.remove(fname + ending)


def rename_file(source, dest):
    """
    Rename file from *source* to *dest*
    If file is a Python script, also rename .pyc and .pyo files if any
    """
    os.rename(source, dest)
    __remove_pyc_pyo(source)


def remove_file(fname):
    """
    Remove file *fname*
    If file is a Python script, also rename .pyc and .pyo files if any
    """
    os.remove(fname)
    __remove_pyc_pyo(fname)


def move_file(source, dest):
    """
    Move file from *source* to *dest*
    If file is a Python script, also rename .pyc and .pyo files if any
    """
    shutil.copy(source, dest)
    remove_file(source)


def abspardir(path):
    """Return absolute parent dir"""
    return osp.abspath(osp.join(path, os.pardir))


def get_common_path(pathlist):
    """Return common path for all paths in pathlist"""
    common = osp.normpath(osp.commonprefix(pathlist))
    if len(common) > 1:
        if not osp.isdir(common):
            return abspardir(common)
        for path in pathlist:
            if not osp.isdir(osp.join(common, path[len(common) + 1:])):
                # `common` is not the real common prefix
                return abspardir(common)
        return osp.abspath(common)
    return None


def path_is_writable(path):
    """Check if given path is writable."""
    path = os.path.abspath(os.path.expanduser(os.path.expandvars(path)))

    path_exists = False
    if os.path.isfile(path):
        test_filepath = path
        remove = False
    else:
        path_exists = os.path.isdir(path)
        remove = True
        if not path_exists:
            try:
                os.makedirs(path)
            except Exception:  # pylint: disable=broad-except
                return False

        test_filepath = os.path.join(path, uuid.uuid4().hex[:5])

    try:
        fh = open(test_filepath, 'a+')  # pylint: disable=consider-using-with,invalid-name,unspecified-encoding
    except (IOError, OSError):
        return False
    else:
        fh.close()
        try:
            if remove:
                os.remove(test_filepath)
            if not path_exists:
                os.rmdir(path)
        except OSError:
            pass
        return True


def save_pid():
    """Save navigator process ID."""
    pid = os.getpid()

    try:
        with open(PIDFILE, 'w') as f:  # pylint: disable=invalid-name,unspecified-encoding
            f.write(str(pid))
    except Exception:  # pylint: disable=broad-except
        pid = None

    return pid


def load_pid():
    """Load navigator process ID."""
    try:
        with open(PIDFILE, 'r') as f:  # pylint: disable=invalid-name,unspecified-encoding
            pid = f.read()
        pid = int(pid)
    except Exception:  # pylint: disable=broad-except
        pid = None

    if pid is not None:
        is_running = psutil.pid_exists(pid)
        process = None
        cmds = []
        try:
            process = psutil.Process(pid)
            if process and is_running:
                cmds = process.cmdline()
        except psutil.NoSuchProcess:
            # print('no such process')
            pass
        except psutil.AccessDenied:
            # Try to remove the pid file, if not possible return False
            if not remove_pid():
                return False

        cmds = [cmd.lower() for cmd in cmds]

        # Check bootstrap
        ch1 = [c for c in cmds if 'python' in c or 'bootstrap.py' in c]
        ch2 = [c for c in cmds if 'python' in c or 'anaconda-navigator' in c]
        ch3 = [c for c in cmds if 'navigator.app' in c]

        check = any(ch1) or any(ch2) or any(ch3)

        if not check:
            pid = None

    return pid


def remove_pid():
    """Load navigator process ID."""
    check = True
    try:
        os.remove(PIDFILE)
    except Exception:  # pylint: disable=broad-except
        check = False
    return check


def remove_lock():
    """Load navigator process ID."""
    check = True
    try:
        os.remove(LOCKFILE)
    except Exception:  # pylint: disable=broad-except
        check = False
    return check


def set_windows_appusermodelid():
    """Make sure correct icon is used on Windows 7 taskbar"""
    try:
        from ctypes import windll  # pylint: disable=import-outside-toplevel
        name = 'anaconda.Anaconda-Navigator'
        return windll.shell32.SetCurrentProcessExplicitAppUserModelID(name)
    except AttributeError:
        return 'SetCurrentProcessExplicitAppUserModelID not found'


if __name__ == '__main__':
    if os.name == 'nt':
        assert get_common_path(  # nosec
            [
                'D:\\Python\\spyder-v21\\spyder\\widgets',
                'D:\\Python\\spyder\\spyder\\utils',
                'D:\\Python\\spyder\\spyder\\widgets',
                'D:\\Python\\spyder-v21\\spyder\\utils',
            ]
        ) == 'D:\\Python'
    else:
        assert get_common_path(  # nosec
            [
                '/Python/spyder-v21/spyder.widgets',
                '/Python/spyder/spyder.utils',
                '/Python/spyder/spyder.widgets',
                '/Python/spyder-v21/spyder.utils',
            ]
        ) == '/Python'
    print(save_pid())
    print(load_pid())
    print(remove_pid())



# -*- coding: utf-8 -*-

"""Core structures for third-party application descriptions."""

__all__ = ['BaseApp', 'BaseWebApp', 'BaseInstallableApp']

import abc
import typing
from anaconda_navigator.utils.constants import AppType

if typing.TYPE_CHECKING:
    import typing_extensions
    from anaconda_navigator.api import process
    from anaconda_navigator.config import user as user_config
    from . import detectors


class BaseApp(metaclass=abc.ABCMeta):  # pylint: disable=too-few-public-methods,too-many-instance-attributes
    """
    Root description of the application.

    :param app_type: Type of the application (web application, installable, etc.).
    :param app_name: Alias of the application in a package-naming format.
    :param display_name: Name of the application to show in home tile.
    :param description: Description of the application to show in home tile.
    :param image_path: Application icon to show in home tile.
    :param config: Application configuration.
    """

    def __init__(  # pylint: disable=too-many-arguments
            self,
            app_type: AppType,
            app_name: str,
            display_name: str,
            description: str,
            image_path: str,
            is_available: bool,
            non_conda: bool,
            config: 'user_config.UserConfig',
    ) -> None:
        """Initialize new :class:`~BaseApp` instance."""
        self.app_type: 'typing_extensions.Final[AppType]' = app_type
        self.app_name: 'typing_extensions.Final[str]' = app_name
        self.display_name: 'typing_extensions.Final[str]' = display_name
        self.description: 'typing_extensions.Final[str]' = description
        self.image_path: 'typing_extensions.Final[str]' = image_path
        self.non_conda: 'typing_extensions.Final[bool]' = non_conda
        self.config: 'typing_extensions.Final[user_config.UserConfig]' = config
        self.is_available: 'typing_extensions.Final[bool]' = is_available


class BaseWebApp(BaseApp, metaclass=abc.ABCMeta):  # pylint: disable=too-few-public-methods
    """
    Common base for all external web applications.

    Most of the params reused from the :class:`~BaseApp`.

    :param url: Web page to open for the application.
    """

    def __init__(  # pylint: disable=too-many-arguments
            self,
            app_name: str,
            display_name: str,
            description: str,
            image_path: str,
            url: str,
            is_available: bool,
            config: 'user_config.UserConfig',
    ) -> None:
        """Initialize new :class:`~BaseWebApp` instance."""
        super().__init__(
            app_type=AppType.WEB,
            app_name=app_name,
            display_name=display_name,
            description=description,
            image_path=image_path,
            is_available=is_available,
            non_conda=True,
            config=config,
        )

        self.url: 'typing_extensions.Final[str]' = url


class BaseInstallableApp(BaseApp, metaclass=abc.ABCMeta):
    """
    Common base for all external installable applications.

    Most of the params reused from the :class:`~BaseApp`.

    :param detector: Detector of the application location.
    :param process_api: API to spawn new processes (for extensions installation etc.).
    """

    def __init__(  # pylint: disable=too-many-arguments
            self,
            app_name: str,
            display_name: str,
            description: str,
            image_path: str,
            detector: 'detectors.Source',
            is_available: bool,
            process_api: 'process.WorkerManager',
            config: 'user_config.UserConfig',
            extra_arguments: typing.Sequence[typing.Any],
    ) -> None:
        """Initialize new :class:`~BaseInstallableApp` instance."""
        super().__init__(
            app_type=AppType.INSTALLABLE,
            app_name=app_name,
            display_name=display_name,
            description=description,
            image_path=image_path,
            is_available=is_available,
            non_conda=True,
            config=config,
        )
        self.extra_arguments: 'typing_extensions.Final[typing.Sequence[typing.Any]]' = extra_arguments
        self._process_api: 'typing_extensions.Final[process.WorkerManager]' = process_api

        location: 'typing.Optional[detectors.DetectedApplication]'
        for location in detector():
            if location.complete:
                break
        else:
            location = None
        self.__location: 'typing_extensions.Final[typing.Optional[detectors.DetectedApplication]]' = location

        self.config.set('main', f'{self.app_name}_path', self.root or '')

    @property
    def executable(self) -> typing.Optional[str]:
        """Return executable command."""
        if self.__location is None:
            return None

        from anaconda_navigator.utils import launch  # pylint: disable=import-outside-toplevel
        return launch.safe_argument(self.__location.executable)

    @property
    def is_installation_enabled(self) -> bool:
        """Application can be installed."""
        return hasattr(self, 'install')

    @property
    def is_installed(self) -> bool:
        """Return whether the app is installed."""
        return bool(self.executable)

    @property
    def root(self) -> typing.Optional[str]:
        """Return application root directory."""
        if self.__location is None:
            return None
        return self.__location.root

    @property
    def version(self) -> typing.Optional[str]:
        """Version of the installed application."""
        if self.__location is None:
            return None
        return self.__location.version

    @property
    def versions(self) -> typing.Sequence[str]:
        """List of available application versions."""
        if self.__location is None:
            return []
        return [self.__location.version]

    @abc.abstractmethod
    def update_config(self, prefix: str) -> None:
        """Update user config to use selected Python prefix interpreter."""

    @abc.abstractmethod
    def install_extensions(self) -> 'process.ProcessWorker':
        """Install app extensions."""



# -*- coding: utf-8 -*-

"""Common interface for third-party applications."""

import typing
from .datalore import *
from .ibm_watson import *
from .pycharm import *
from .vscode import *

if typing.TYPE_CHECKING:
    import typing_extensions
    from anaconda_navigator.config import user as user_config
    from anaconda_navigator.api import process
    from .base import *

    class InstallableAppInitializer(typing_extensions.Protocol):  # pylint: disable=too-few-public-methods
        """Common interface for installable apps initializer."""

        def __call__(
                self,
                config: user_config.UserConfig,
                process_api: process.WorkerManager,
        ) -> BaseInstallableApp:
            """Initialize new installable app."""

    class WebAppInitializer(typing_extensions.Protocol):  # pylint: disable=too-few-public-methods
        """Common interface for web apps initializer."""

        def __call__(
                self,
                config: user_config.UserConfig,
        ) -> BaseWebApp:
            """Initialize new web app."""


apps: 'typing_extensions.Final[typing.Mapping[str, InstallableAppInitializer]]' = {
    'pycharm_ce': PyCharmCEApp,
    'pycharm_pro': PyCharmProApp,
    'vscode': VSCodeApp,
}
web_apps: 'typing_extensions.Final[typing.Mapping[str, WebAppInitializer]]' = {
    'datalore': DataloreApp,
    'ibm_watson': IBMWatsonApp,
}



#!/opt/concourse/worker/volumes/live/f42d4aad-1b9c-446f-441c-1fabebea4417/volume/anaconda-navigator_1648477375444/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/Anaconda-Navigator.app/Contents/MacOS/python
from subprocess import call

call(["/opt/concourse/worker/volumes/live/f42d4aad-1b9c-446f-441c-1fabebea4417/volume/anaconda-navigator_1648477375444/_h_env_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placehold_placeh/bin/anaconda-navigator"])



# -*- coding: utf-8 -*-

# pylint: disable=invalid-name

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Widgets to list environemnts available to edit in the Environments tab."""

from __future__ import absolute_import, division, print_function

import sys
import typing
from qtpy.QtCore import QSize, Qt, Signal  # pylint: disable=no-name-in-module
from qtpy.QtWidgets import QHBoxLayout, QPushButton  # pylint: disable=no-name-in-module
from anaconda_navigator.utils.qthelpers import qapplication
from anaconda_navigator.utils.styles import SASS_VARIABLES
from anaconda_navigator.widgets import ButtonEnvironmentOptions, FrameBase
from anaconda_navigator.widgets.lists import ListWidgetBase, ListWidgetItemBase
from anaconda_navigator.widgets.spinner import NavigatorSpinner


# --- Widgets used in styling
# -----------------------------------------------------------------------------

class WidgetEnvironment(FrameBase):
    """Main list item widget used in CSS styling."""

    sig_entered = Signal()
    sig_left = Signal()
    clicked = Signal()

    def __init__(self, *args, **kwargs):
        """Main list item widget used in CSS styling."""
        super().__init__(*args, **kwargs)
        self._hover = False

    def keyPressEvent(self, event):
        """Override Qt method."""
        key = event.key()
        if key in [Qt.Key_Return, Qt.Key_Enter, Qt.Key_Space]:
            self.clicked.emit()
        super().keyPressEvent(event)

    def enterEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self._hover = True
        self.sig_entered.emit()

    def leaveEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self._hover = False
        self.sig_left.emit()

    def mouseReleaseEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        if self._hover:
            self.clicked.emit()


class ButtonEnvironmentName(QPushButton):
    """Button used in CSS styling."""

    sig_entered = Signal()
    sig_left = Signal()

    def focusInEvent(self, event):
        """Override Qt method."""
        super().focusInEvent(event)
        self.sig_entered.emit()

    def focusOutEvent(self, event):
        """Override Qt method."""
        super().focusOutEvent(event)
        self.sig_left.emit()

    def enterEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self.sig_entered.emit()

    def leaveEvent(self, event):
        """Override Qt method."""
        super().enterEvent(event)
        self.sig_left.emit()

    def setProperty(self, name, value):
        """Override Qt method."""
        QPushButton.setProperty(self, name, value)
        self.style().unpolish(self)
        self.style().polish(self)
        self.update()

    def set_selected(self, value):
        """Set selected status."""
        self._selected = value  # pylint: disable=attribute-defined-outside-init
        self.setProperty('pressed', value)

    def set_hovered(self, value):
        """Set hovered status."""
        self.setProperty('hovered', value)


class FrameEnvironmentIcon(FrameBase):
    """Label used in CSS styling."""


# --- Main widgets
# -----------------------------------------------------------------------------


class BaseListWidgetEnv(ListWidgetBase):
    """Custom list widget environments."""
    def ordered_widgets(self):
        """Return a list of the ordered widgets."""
        ordered_widgets = []
        for item in self.items():
            ordered_widgets.extend(item.ordered_widgets())
        return ordered_widgets

    def setup_item(self, item):
        """Add additional logic after adding an item."""
        index = self._items.index(item)

        item.widget.clicked.connect(lambda v=None, i=index: self.setCurrentRow(i, loading=True))
        item.widget.clicked.connect(lambda v=None, it=item: self.sig_item_selected.emit(it))
        item.widget.sig_entered.connect(lambda v=None: item.set_hovered(True))
        item.button_name.sig_entered.connect(lambda v=None: self.scroll_to_item(item))
        item.widget.sig_left.connect(lambda v=None: item.set_hovered(False))
        item.button_name.clicked.connect(lambda v=None, it=item: self.sig_item_selected.emit(it))
        item.button_name.clicked.connect(lambda v=None, i=index: self.setCurrentRow(i, loading=True))
        item.button_name.sig_entered.connect(lambda v=None: item.set_hovered(True))
        item.button_name.sig_left.connect(lambda v=None: item.set_hovered(False))

        fm = item.button_name.fontMetrics()
        elided_name = fm.elidedText(item.name, Qt.ElideRight, 150)
        item.button_name.setText(elided_name)

    def text(self):
        """
        Override Qt Method.

        Return full name, not elided text.
        """
        item = self.currentItem()
        text = item.name if item else str()
        return text


class BaseListItemEnv(ListWidgetItemBase):
    """Widget to build an item for the environments list."""
    ENV_ITEM_WIDTH: typing.ClassVar[int] = SASS_VARIABLES.WIDGET_ENVIRONMENT_TOTAL_WIDTH
    ENV_ITEM_HEIGHT: typing.ClassVar[int] = SASS_VARIABLES.WIDGET_ENVIRONMENT_TOTAL_HEIGHT

    def __init__(self, name=None, prefix=None):
        """Widget to build an item for the environments list."""
        super().__init__()

        self._selected = False
        self._name = name
        self._prefix = prefix

        # Widgets
        self.button_name = ButtonEnvironmentName(name)
        self.widget = WidgetEnvironment()
        self.widget.button_name = self.button_name
        self.spinner = NavigatorSpinner(parent=self.widget, total_width=20)

        # Widget setup
        self.button_name.setDefault(True)
        self.widget.setFocusPolicy(Qt.NoFocus)
        self.button_name.setFocusPolicy(Qt.StrongFocus)
        self.button_name.setToolTip(prefix if prefix else '')

        # Layouts
        layout = self._get_env_item_layout()

        self.widget.setLayout(layout)
        self.setSizeHint(self.widget.sizeHint())

    def _get_env_item_layout(self):
        layout = QHBoxLayout()
        layout.addWidget(self.button_name)
        layout.addStretch()

        return layout

    def ordered_widgets(self):
        """Return a list of the ordered widgets."""
        return [self.button_name]

    @property
    def name(self):
        """Resturn the environment name."""
        return self._name

    @property
    def prefix(self):
        """Return the environment prefix."""
        return self._prefix

    def set_hovered(self, value):
        """Set widget as hovered."""
        self.widget.setProperty('hovered', value)
        self.button_name.set_hovered(value)

    def set_selected(self, value):
        """Set widget as selected."""
        self._selected = value
        try:
            self.widget.setProperty('pressed', value)
            self.button_name.set_selected(value)
        except RuntimeError:
            pass

        self.button_name.setDisabled(value)

    @classmethod
    def widget_size(cls):
        """Return the size defined in the SASS file."""
        return QSize(cls.ENV_ITEM_WIDTH, cls.ENV_ITEM_HEIGHT)


class ListWidgetEnv(BaseListWidgetEnv):
    """Custom list widget environments."""

    def setup_item(self, item):
        """Add additional logic after adding an item."""
        super().setup_item(item)
        item.button_options.sig_entered.connect(lambda v=None: self.scroll_to_item(item))


class ListItemEnv(BaseListItemEnv):
    """Widget to build an item for the environments list."""
    def __init__(self, name=None, prefix=None):
        """Widget to build an item for the environments list."""
        super().__init__(name=name, prefix=prefix)

    def _get_env_item_layout(self):
        self.button_options = ButtonEnvironmentOptions()
        self.widget.button_options = self.button_options  # pylint: disable=attribute-defined-outside-init

        self.button_options.setFocusPolicy(Qt.StrongFocus)

        self.frame_icon = FrameEnvironmentIcon()

        lay = QHBoxLayout()
        lay.addWidget(self.spinner)
        self.frame_icon.setLayout(lay)

        layout = QHBoxLayout()
        layout.addWidget(self.frame_icon)
        layout.addWidget(self.button_name)
        layout.addStretch()
        layout.addWidget(self.button_options)
        layout.addSpacing(16)

        return layout

    def ordered_widgets(self):
        """Return a list of the ordered widgets."""
        return [self.button_name, self.button_options]

    def set_loading(self, value):
        """Set loading status of widget."""
        if value:
            self.spinner.start()
        else:
            self.spinner.stop()

    def set_selected(self, value):
        """Set widget as selected."""
        super().set_selected(value)

        self.spinner.stop()
        if value:
            self.button_options.setVisible(True)
        else:
            self.button_options.setVisible(False)


# --- Local testing
# -----------------------------------------------------------------------------
def print_selected(item):
    """Print selected item name."""
    print('Item selected: ', item.name if item else 'None')


def local_test():  # pragma: no cover
    """Run local test."""
    app = qapplication()
    widget = ListWidgetEnv()
    widget.sig_item_selected.connect(print_selected)
    widget.addItem(ListItemEnv('root'))

    item = ListItemEnv('extralongnamewithsome1212331231321233')
    widget.addItem(item)
    for i in range(5):
        item = ListItemEnv('env ' + str(i))
        widget.addItem(item)

    widget.update_style_sheet()
    widget.show()
    sys.exit(app.exec_())


if __name__ == '__main__':  # pragma: no cover
    local_test()
